"""ActionBatchAppliance API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

import urllib.parse
from typing import Any

from meraki_client.schemas import (
    BulkOrganizationApplianceDnsLocalProfilesAssignmentsCreateItemsItem,
    CreateNetworkAppliancePrefixesDelegatedStaticOrigin,
    CreateNetworkApplianceRfProfileFiveGhzSettings,
    CreateNetworkApplianceRfProfilePerSsidSettings,
    CreateNetworkApplianceRfProfileTwoFourGhzSettings,
    CreateNetworkApplianceVlanDhcpOptionsItem,
    CreateNetworkApplianceVlanIpv6,
    CreateNetworkApplianceVlanMandatoryDhcp,
    CreateOrganizationActionBatchActionsItem,
    CreateOrganizationApplianceDnsLocalProfilesAssignmentsBulkDeleteItemsItem,
    CreateOrganizationApplianceDnsLocalRecordProfile,
    CreateOrganizationApplianceDnsSplitProfileNameservers,
    CreateOrganizationApplianceDnsSplitProfilesAssignmentsBulkCreateItemsItem,
    CreateOrganizationApplianceDnsSplitProfilesAssignmentsBulkDeleteItemsItem,
    UpdateDeviceApplianceRadioSettingsFiveGhzSettings,
    UpdateDeviceApplianceRadioSettingsTwoFourGhzSettings,
    UpdateDeviceApplianceUplinksSettingsInterfaces,
    UpdateNetworkApplianceConnectivityMonitoringDestinationsDestinationsItem,
    UpdateNetworkApplianceFirewallL7FirewallRulesRulesItem,
    UpdateNetworkApplianceFirewallMulticastForwardingRulesItem,
    UpdateNetworkAppliancePrefixesDelegatedStaticOrigin,
    UpdateNetworkApplianceRfProfileFiveGhzSettings,
    UpdateNetworkApplianceRfProfilePerSsidSettings,
    UpdateNetworkApplianceRfProfileTwoFourGhzSettings,
    UpdateNetworkApplianceSdwanInternetPoliciesWanTrafficUplinkPreferencesItem,
    UpdateNetworkApplianceSettingsDynamicDns,
    UpdateNetworkApplianceSingleLanIpv6,
    UpdateNetworkApplianceSingleLanMandatoryDhcp,
    UpdateNetworkApplianceSsidDhcpEnforcedDeauthentication,
    UpdateNetworkApplianceSsidDot11w,
    UpdateNetworkApplianceSsidRadiusServersItem,
    UpdateNetworkApplianceTrafficShapingRulesRulesItem,
    UpdateNetworkApplianceTrafficShapingUplinkBandwidthBandwidthLimits,
    UpdateNetworkApplianceTrafficShapingUplinkSelectionFailoverAndFailback,
    UpdateNetworkApplianceTrafficShapingUplinkSelectionVpnTrafficUplinkPreferencesItem,
    UpdateNetworkApplianceTrafficShapingUplinkSelectionWanTrafficUplinkPreferencesItem,
    UpdateNetworkApplianceTrafficShapingVpnExclusionsCustomItem,
    UpdateNetworkApplianceTrafficShapingVpnExclusionsMajorApplicationsItem,
    UpdateNetworkApplianceVlanDhcpOptionsItem,
    UpdateNetworkApplianceVlanIpv6,
    UpdateNetworkApplianceVlanMandatoryDhcp,
    UpdateNetworkApplianceVlanReservedIpRangesItem,
    UpdateNetworkApplianceVpnBgpNeighborsItem,
    UpdateNetworkApplianceVpnSiteToSiteVpnHubsItem,
    UpdateNetworkApplianceVpnSiteToSiteVpnSubnet,
    UpdateNetworkApplianceVpnSiteToSiteVpnSubnetsItem,
    UpdateOrganizationApplianceDnsLocalRecordProfile,
    UpdateOrganizationApplianceDnsSplitProfileNameservers,
    UpdateOrganizationApplianceVpnSiteToSiteIpsecPeersSlasItemsItem,
    UpdateOrganizationApplianceVpnThirdPartyVPNPeersPeersItem,
)
from meraki_client.types import (
    CreateNetworkApplianceVlanDhcpHandling,
    CreateNetworkApplianceVlanDhcpLeaseTime,
    CreateNetworkApplianceVlanTemplateVlanType,
    UpdateNetworkApplianceSettingsClientTrackingMethod,
    UpdateNetworkApplianceSettingsDeploymentMode,
    UpdateNetworkApplianceSsidAuthMode,
    UpdateNetworkApplianceSsidEncryptionMode,
    UpdateNetworkApplianceSsidWpaEncryptionMode,
    UpdateNetworkApplianceVlanDhcpHandling,
    UpdateNetworkApplianceVlanDhcpLeaseTime,
    UpdateNetworkApplianceVlanTemplateVlanType,
    UpdateNetworkApplianceVpnSiteToSiteVpnMode,
)


class ActionBatchAppliance:
    """ActionBatchAppliance class."""

    def __init__(self) -> None:
        pass

    def update_device_appliance_radio_settings(
        self,
        serial: str,
        *,
        rf_profile_id: str | None = None,
        two_four_ghz_settings: UpdateDeviceApplianceRadioSettingsTwoFourGhzSettings | None = None,
        five_ghz_settings: UpdateDeviceApplianceRadioSettingsFiveGhzSettings | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the radio settings of an appliance.

        [API documentation: updateDeviceApplianceRadioSettings](https://developer.cisco.com/meraki/api-v1/#!update-device-appliance-radio-settings)

        Args:
            serial: Serial.
            rf_profile_id: The ID of an RF profile to assign to the device. If the value of this
                parameter is null, the appropriate basic RF profile (indoor or outdoor)
                will be assigned to the device. Assigning an RF profile will clear ALL
                manually configured overrides on the device (channel width, channel,
                power).
            two_four_ghz_settings: Manual radio settings for 2.4 GHz.
            five_ghz_settings: Manual radio settings for 5 GHz.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/appliance/radio/settings"

        payload: dict[str, Any] = {}
        if rf_profile_id is not None:
            payload["rfProfileId"] = rf_profile_id
        if two_four_ghz_settings is not None:
            payload["twoFourGhzSettings"] = two_four_ghz_settings.model_dump(
                by_alias=True, exclude_none=True
            )
        if five_ghz_settings is not None:
            payload["fiveGhzSettings"] = five_ghz_settings.model_dump(
                by_alias=True, exclude_none=True
            )

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_device_appliance_uplinks_settings(
        self, *, serial: str, interfaces: UpdateDeviceApplianceUplinksSettingsInterfaces
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the uplink settings for an MX appliance.

        [API documentation: updateDeviceApplianceUplinksSettings](https://developer.cisco.com/meraki/api-v1/#!update-device-appliance-uplinks-settings)

        Args:
            serial: Serial.
            interfaces: Interface settings.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/appliance/uplinks/settings"

        payload: dict[str, Any] = {}
        if interfaces is not None:
            payload["interfaces"] = interfaces.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def create_device_appliance_vmx_authentication_token(
        self, serial: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Generate a new vMX authentication token.

        [API documentation: createDeviceApplianceVmxAuthenticationToken](https://developer.cisco.com/meraki/api-v1/#!create-device-appliance-vmx-authentication-token)

        Args:
            serial: Serial.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/appliance/vmx/authenticationToken"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
        )

    def update_network_appliance_connectivity_monitoring_destinations(
        self,
        network_id: str,
        *,
        destinations: list[UpdateNetworkApplianceConnectivityMonitoringDestinationsDestinationsItem]
        | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the connectivity testing destinations for an MX network.

        [API documentation: updateNetworkApplianceConnectivityMonitoringDestinations](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-connectivity-monitoring-destinations)

        Args:
            network_id: Network ID.
            destinations: The list of connectivity monitoring destinations.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/connectivityMonitoringDestinations"

        payload: dict[str, Any] = {}
        if destinations is not None:
            payload["destinations"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in destinations
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_firewall_l7_firewall_rules(
        self,
        network_id: str,
        *,
        rules: list[UpdateNetworkApplianceFirewallL7FirewallRulesRulesItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the MX L7 firewall rules for an MX network.

        [API documentation: updateNetworkApplianceFirewallL7FirewallRules](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-firewall-l-7-firewall-rules)

        Args:
            network_id: Network ID.
            rules: An ordered array of the MX L7 firewall rules.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/firewall/l7FirewallRules"

        payload: dict[str, Any] = {}
        if rules is not None:
            payload["rules"] = [item.model_dump(by_alias=True, exclude_none=True) for item in rules]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_firewall_multicast_forwarding(
        self,
        *,
        network_id: str,
        rules: list[UpdateNetworkApplianceFirewallMulticastForwardingRulesItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update static multicast forward rules for a network.

        [API documentation: updateNetworkApplianceFirewallMulticastForwarding](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-firewall-multicast-forwarding)

        Args:
            network_id: Network ID.
            rules: Static multicast forwarding rules. Pass an empty array to clear all rules.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/firewall/multicastForwarding"

        payload: dict[str, Any] = {}
        if rules is not None:
            payload["rules"] = [item.model_dump(by_alias=True, exclude_none=True) for item in rules]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_port(
        self,
        *,
        network_id: str,
        port_id: str,
        enabled: bool | None = None,
        drop_untagged_traffic: bool | None = None,
        type_: str | None = None,
        vlan: int | None = None,
        allowed_vlans: str | None = None,
        access_policy: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the per-port VLAN settings for a single MX port.

        [API documentation: updateNetworkAppliancePort](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-port)

        Args:
            network_id: Network ID.
            port_id: Port ID.
            enabled: The status of the port.
            drop_untagged_traffic: Trunk port can Drop all Untagged traffic. When true, no VLAN is
                required. Access ports cannot have dropUntaggedTraffic set to true.
            type_: The type of the port: 'access' or 'trunk'.
            vlan: Native VLAN when the port is in Trunk mode. Access VLAN when the port is in Access
                mode.
            allowed_vlans: Comma-delimited list of the VLAN ID's allowed on the port, or 'all' to
                permit all VLAN's on the port.
            access_policy: The name of the policy. Only applicable to Access ports. Valid values
                are: 'open', '8021x-radius', 'mac-radius', 'hybris-radius' for MX64 or
                Z3 or any MX supporting the per port authentication feature. Otherwise,
                'open' is the only valid value and 'open' is the default value if the
                field is missing.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        port_id = urllib.parse.quote(str(port_id), safe="")
        path = f"/networks/{network_id}/appliance/ports/{port_id}"

        payload: dict[str, Any] = {}
        if enabled is not None:
            payload["enabled"] = enabled
        if drop_untagged_traffic is not None:
            payload["dropUntaggedTraffic"] = drop_untagged_traffic
        if type_ is not None:
            payload["type"] = type_
        if vlan is not None:
            payload["vlan"] = vlan
        if allowed_vlans is not None:
            payload["allowedVlans"] = allowed_vlans
        if access_policy is not None:
            payload["accessPolicy"] = access_policy

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def create_network_appliance_prefixes_delegated_static(
        self,
        *,
        network_id: str,
        prefix: str,
        origin: CreateNetworkAppliancePrefixesDelegatedStaticOrigin,
        description: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Add a static delegated prefix from a network.

        [API documentation: createNetworkAppliancePrefixesDelegatedStatic](https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-prefixes-delegated-static)

        Args:
            network_id: Network ID.
            prefix: A static IPv6 prefix.
            origin: The origin of the prefix.
            description: A name or description for the prefix.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/prefixes/delegated/statics"

        payload: dict[str, Any] = {}
        if prefix is not None:
            payload["prefix"] = prefix
        if origin is not None:
            payload["origin"] = origin.model_dump(by_alias=True, exclude_none=True)
        if description is not None:
            payload["description"] = description

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_network_appliance_prefixes_delegated_static(
        self,
        *,
        network_id: str,
        static_delegated_prefix_id: str,
        prefix: str | None = None,
        origin: UpdateNetworkAppliancePrefixesDelegatedStaticOrigin | None = None,
        description: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a static delegated prefix from a network.

        [API documentation: updateNetworkAppliancePrefixesDelegatedStatic](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-prefixes-delegated-static)

        Args:
            network_id: Network ID.
            static_delegated_prefix_id: Static delegated prefix ID.
            prefix: A static IPv6 prefix.
            origin: The origin of the prefix.
            description: A name or description for the prefix.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        static_delegated_prefix_id = urllib.parse.quote(str(static_delegated_prefix_id), safe="")
        path = f"/networks/{network_id}/appliance/prefixes/delegated/statics/{static_delegated_prefix_id}"

        payload: dict[str, Any] = {}
        if prefix is not None:
            payload["prefix"] = prefix
        if origin is not None:
            payload["origin"] = origin.model_dump(by_alias=True, exclude_none=True)
        if description is not None:
            payload["description"] = description

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_network_appliance_prefixes_delegated_static(
        self, *, network_id: str, static_delegated_prefix_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a static delegated prefix from a network.

        [API documentation: deleteNetworkAppliancePrefixesDelegatedStatic](https://developer.cisco.com/meraki/api-v1/#!delete-network-appliance-prefixes-delegated-static)

        Args:
            network_id: Network ID.
            static_delegated_prefix_id: Static delegated prefix ID.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        static_delegated_prefix_id = urllib.parse.quote(str(static_delegated_prefix_id), safe="")
        path = f"/networks/{network_id}/appliance/prefixes/delegated/statics/{static_delegated_prefix_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_network_appliance_rf_profile(
        self,
        *,
        network_id: str,
        name: str,
        two_four_ghz_settings: CreateNetworkApplianceRfProfileTwoFourGhzSettings | None = None,
        five_ghz_settings: CreateNetworkApplianceRfProfileFiveGhzSettings | None = None,
        per_ssid_settings: CreateNetworkApplianceRfProfilePerSsidSettings | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Creates new RF profile for this network.

        [API documentation: createNetworkApplianceRfProfile](https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-rf-profile)

        Args:
            network_id: Network ID.
            name: The name of the new profile. Must be unique. This param is required on creation.
            two_four_ghz_settings: Settings related to 2.4Ghz band.
            five_ghz_settings: Settings related to 5Ghz band.
            per_ssid_settings: Per-SSID radio settings by number.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/rfProfiles"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if two_four_ghz_settings is not None:
            payload["twoFourGhzSettings"] = two_four_ghz_settings.model_dump(
                by_alias=True, exclude_none=True
            )
        if five_ghz_settings is not None:
            payload["fiveGhzSettings"] = five_ghz_settings.model_dump(
                by_alias=True, exclude_none=True
            )
        if per_ssid_settings is not None:
            payload["perSsidSettings"] = per_ssid_settings.model_dump(
                by_alias=True, exclude_none=True
            )

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_network_appliance_rf_profile(
        self,
        *,
        network_id: str,
        rf_profile_id: str,
        name: str | None = None,
        two_four_ghz_settings: UpdateNetworkApplianceRfProfileTwoFourGhzSettings | None = None,
        five_ghz_settings: UpdateNetworkApplianceRfProfileFiveGhzSettings | None = None,
        per_ssid_settings: UpdateNetworkApplianceRfProfilePerSsidSettings | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updates specified RF profile for this network.

        [API documentation: updateNetworkApplianceRfProfile](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-rf-profile)

        Args:
            network_id: Network ID.
            rf_profile_id: Rf profile ID.
            name: The name of the new profile. Must be unique.
            two_four_ghz_settings: Settings related to 2.4Ghz band.
            five_ghz_settings: Settings related to 5Ghz band.
            per_ssid_settings: Per-SSID radio settings by number.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        rf_profile_id = urllib.parse.quote(str(rf_profile_id), safe="")
        path = f"/networks/{network_id}/appliance/rfProfiles/{rf_profile_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if two_four_ghz_settings is not None:
            payload["twoFourGhzSettings"] = two_four_ghz_settings.model_dump(
                by_alias=True, exclude_none=True
            )
        if five_ghz_settings is not None:
            payload["fiveGhzSettings"] = five_ghz_settings.model_dump(
                by_alias=True, exclude_none=True
            )
        if per_ssid_settings is not None:
            payload["perSsidSettings"] = per_ssid_settings.model_dump(
                by_alias=True, exclude_none=True
            )

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_network_appliance_rf_profile(
        self, *, network_id: str, rf_profile_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a RF Profile.

        [API documentation: deleteNetworkApplianceRfProfile](https://developer.cisco.com/meraki/api-v1/#!delete-network-appliance-rf-profile)

        Args:
            network_id: Network ID.
            rf_profile_id: Rf profile ID.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        rf_profile_id = urllib.parse.quote(str(rf_profile_id), safe="")
        path = f"/networks/{network_id}/appliance/rfProfiles/{rf_profile_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def update_network_appliance_sdwan_internet_policies(
        self,
        network_id: str,
        *,
        wan_traffic_uplink_preferences: list[
            UpdateNetworkApplianceSdwanInternetPoliciesWanTrafficUplinkPreferencesItem
        ]
        | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update SDWAN internet traffic preferences for an MX network.

        [API documentation: updateNetworkApplianceSdwanInternetPolicies](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-sdwan-internet-policies)

        Args:
            network_id: Network ID.
            wan_traffic_uplink_preferences: policies with respective traffic filters for an MX
                network.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/sdwan/internetPolicies"

        payload: dict[str, Any] = {}
        if wan_traffic_uplink_preferences is not None:
            payload["wanTrafficUplinkPreferences"] = [
                item.model_dump(by_alias=True, exclude_none=True)
                for item in wan_traffic_uplink_preferences
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_settings(
        self,
        network_id: str,
        *,
        client_tracking_method: UpdateNetworkApplianceSettingsClientTrackingMethod | None = None,
        deployment_mode: UpdateNetworkApplianceSettingsDeploymentMode | None = None,
        dynamic_dns: UpdateNetworkApplianceSettingsDynamicDns | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the appliance settings for a network.

        [API documentation: updateNetworkApplianceSettings](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-settings)

        Args:
            network_id: Network ID.
            client_tracking_method: Client tracking method of a network.
            deployment_mode: Deployment mode of a network.
            dynamic_dns: Dynamic DNS settings for a network.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/settings"

        payload: dict[str, Any] = {}
        if client_tracking_method is not None:
            payload["clientTrackingMethod"] = client_tracking_method
        if deployment_mode is not None:
            payload["deploymentMode"] = deployment_mode
        if dynamic_dns is not None:
            payload["dynamicDns"] = dynamic_dns.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_single_lan(
        self,
        network_id: str,
        *,
        subnet: str | None = None,
        appliance_ip: str | None = None,
        ipv6: UpdateNetworkApplianceSingleLanIpv6 | None = None,
        mandatory_dhcp: UpdateNetworkApplianceSingleLanMandatoryDhcp | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update single LAN configuration.

        [API documentation: updateNetworkApplianceSingleLan](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-single-lan)

        Args:
            network_id: Network ID.
            subnet: The subnet of the single LAN configuration.
            appliance_ip: The appliance IP address of the single LAN.
            ipv6: IPv6 configuration on the VLAN.
            mandatory_dhcp: Mandatory DHCP will enforce that clients connecting to this LAN must use
                the IP address assigned by the DHCP server. Clients who use a static IP
                address won't be able to associate. Only available on firmware versions
                17.0 and above.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/singleLan"

        payload: dict[str, Any] = {}
        if subnet is not None:
            payload["subnet"] = subnet
        if appliance_ip is not None:
            payload["applianceIp"] = appliance_ip
        if ipv6 is not None:
            payload["ipv6"] = ipv6.model_dump(by_alias=True, exclude_none=True)
        if mandatory_dhcp is not None:
            payload["mandatoryDhcp"] = mandatory_dhcp.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_ssid(
        self,
        *,
        network_id: str,
        number: str,
        name: str | None = None,
        enabled: bool | None = None,
        default_vlan_id: int | None = None,
        auth_mode: UpdateNetworkApplianceSsidAuthMode | None = None,
        psk: str | None = None,
        radius_servers: list[UpdateNetworkApplianceSsidRadiusServersItem] | None = None,
        encryption_mode: UpdateNetworkApplianceSsidEncryptionMode | None = None,
        wpa_encryption_mode: UpdateNetworkApplianceSsidWpaEncryptionMode | None = None,
        visible: bool | None = None,
        dhcp_enforced_deauthentication: UpdateNetworkApplianceSsidDhcpEnforcedDeauthentication
        | None = None,
        dot11w: UpdateNetworkApplianceSsidDot11w | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the attributes of an MX SSID.

        [API documentation: updateNetworkApplianceSsid](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-ssid)

        Args:
            network_id: Network ID.
            number: Number.
            name: The name of the SSID.
            enabled: Whether or not the SSID is enabled.
            default_vlan_id: The VLAN ID of the VLAN associated to this SSID. This parameter is only
                valid if the network is in routed mode.
            auth_mode: The association control method for the SSID ('open', 'psk', '8021x-meraki' or
                '8021x-radius').
            psk: The passkey for the SSID. This param is only valid if the authMode is 'psk'.
            radius_servers: The RADIUS 802.1x servers to be used for authentication. This param is
                only valid if the authMode is '8021x-radius'.
            encryption_mode: The psk encryption mode for the SSID ('wep' or 'wpa'). This param is
                only valid if the authMode is 'psk'.
            wpa_encryption_mode: The types of WPA encryption. ('WPA1 and WPA2', 'WPA2 only', 'WPA3
                Transition Mode' or 'WPA3 only'). This param is only valid if (1) the
                authMode is 'psk' & the encryptionMode is 'wpa' OR (2) the authMode is
                '8021x-meraki' OR (3) the authMode is '8021x-radius'.
            visible: Boolean indicating whether the MX should advertise or hide this SSID.
            dhcp_enforced_deauthentication: DHCP Enforced Deauthentication enables the
                disassociation of wireless clients in addition to Mandatory DHCP. This
                param is only valid on firmware versions >= MX 17.0 where the associated
                LAN has Mandatory DHCP Enabled.
            dot11w: The current setting for Protected Management Frames (802.11w).

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        number = urllib.parse.quote(str(number), safe="")
        path = f"/networks/{network_id}/appliance/ssids/{number}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if enabled is not None:
            payload["enabled"] = enabled
        if default_vlan_id is not None:
            payload["defaultVlanId"] = default_vlan_id
        if auth_mode is not None:
            payload["authMode"] = auth_mode
        if psk is not None:
            payload["psk"] = psk
        if radius_servers is not None:
            payload["radiusServers"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in radius_servers
            ]
        if encryption_mode is not None:
            payload["encryptionMode"] = encryption_mode
        if wpa_encryption_mode is not None:
            payload["wpaEncryptionMode"] = wpa_encryption_mode
        if visible is not None:
            payload["visible"] = visible
        if dhcp_enforced_deauthentication is not None:
            payload["dhcpEnforcedDeauthentication"] = dhcp_enforced_deauthentication.model_dump(
                by_alias=True, exclude_none=True
            )
        if dot11w is not None:
            payload["dot11w"] = dot11w.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def create_network_appliance_traffic_shaping_custom_performance_class(
        self,
        *,
        network_id: str,
        name: str,
        max_latency: int | None = None,
        max_jitter: int | None = None,
        max_loss_percentage: int | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Add a custom performance class for an MX network.

        [API documentation: createNetworkApplianceTrafficShapingCustomPerformanceClass](https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-traffic-shaping-custom-performance-class)

        Args:
            network_id: Network ID.
            name: Name of the custom performance class.
            max_latency: Maximum latency in milliseconds.
            max_jitter: Maximum jitter in milliseconds.
            max_loss_percentage: Maximum percentage of packet loss.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/trafficShaping/customPerformanceClasses"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if max_latency is not None:
            payload["maxLatency"] = max_latency
        if max_jitter is not None:
            payload["maxJitter"] = max_jitter
        if max_loss_percentage is not None:
            payload["maxLossPercentage"] = max_loss_percentage

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_network_appliance_traffic_shaping_custom_performance_class(
        self,
        *,
        network_id: str,
        custom_performance_class_id: str,
        name: str | None = None,
        max_latency: int | None = None,
        max_jitter: int | None = None,
        max_loss_percentage: int | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a custom performance class for an MX network.

        [API documentation: updateNetworkApplianceTrafficShapingCustomPerformanceClass](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-traffic-shaping-custom-performance-class)

        Args:
            network_id: Network ID.
            custom_performance_class_id: Custom performance class ID.
            name: Name of the custom performance class.
            max_latency: Maximum latency in milliseconds.
            max_jitter: Maximum jitter in milliseconds.
            max_loss_percentage: Maximum percentage of packet loss.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        custom_performance_class_id = urllib.parse.quote(str(custom_performance_class_id), safe="")
        path = f"/networks/{network_id}/appliance/trafficShaping/customPerformanceClasses/{custom_performance_class_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if max_latency is not None:
            payload["maxLatency"] = max_latency
        if max_jitter is not None:
            payload["maxJitter"] = max_jitter
        if max_loss_percentage is not None:
            payload["maxLossPercentage"] = max_loss_percentage

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_network_appliance_traffic_shaping_custom_performance_class(
        self, *, network_id: str, custom_performance_class_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a custom performance class from an MX network.

        [API documentation: deleteNetworkApplianceTrafficShapingCustomPerformanceClass](https://developer.cisco.com/meraki/api-v1/#!delete-network-appliance-traffic-shaping-custom-performance-class)

        Args:
            network_id: Network ID.
            custom_performance_class_id: Custom performance class ID.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        custom_performance_class_id = urllib.parse.quote(str(custom_performance_class_id), safe="")
        path = f"/networks/{network_id}/appliance/trafficShaping/customPerformanceClasses/{custom_performance_class_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def update_network_appliance_traffic_shaping_rules(
        self,
        network_id: str,
        *,
        default_rules_enabled: bool | None = None,
        rules: list[UpdateNetworkApplianceTrafficShapingRulesRulesItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the traffic shaping settings rules for an MX network.

        [API documentation: updateNetworkApplianceTrafficShapingRules](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-traffic-shaping-rules)

        Args:
            network_id: Network ID.
            default_rules_enabled: Whether default traffic shaping rules are enabled (true) or
                disabled (false). There are 4 default rules, which can be seen on your
                network's traffic shaping page. Note that default rules count against
                the rule limit of 8.
            rules: An array of traffic shaping rules. Rules are applied in the order that they are
                specified in. An empty list (or null) means no rules. Note that you are
                allowed a maximum of 8 rules.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/trafficShaping/rules"

        payload: dict[str, Any] = {}
        if default_rules_enabled is not None:
            payload["defaultRulesEnabled"] = default_rules_enabled
        if rules is not None:
            payload["rules"] = [item.model_dump(by_alias=True, exclude_none=True) for item in rules]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_traffic_shaping_uplink_bandwidth(
        self,
        network_id: str,
        *,
        bandwidth_limits: UpdateNetworkApplianceTrafficShapingUplinkBandwidthBandwidthLimits
        | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updates the uplink bandwidth settings for your MX network.

        [API documentation: updateNetworkApplianceTrafficShapingUplinkBandwidth](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-traffic-shaping-uplink-bandwidth)

        Args:
            network_id: Network ID.
            bandwidth_limits: A mapping of uplinks to their bandwidth settings (be sure to check
                which uplinks are supported for your network).

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/trafficShaping/uplinkBandwidth"

        payload: dict[str, Any] = {}
        if bandwidth_limits is not None:
            payload["bandwidthLimits"] = bandwidth_limits.model_dump(
                by_alias=True, exclude_none=True
            )

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_traffic_shaping_uplink_selection(
        self,
        network_id: str,
        *,
        active_active_auto_vpn_enabled: bool | None = None,
        default_uplink: str | None = None,
        load_balancing_enabled: bool | None = None,
        failover_and_failback: UpdateNetworkApplianceTrafficShapingUplinkSelectionFailoverAndFailback
        | None = None,
        wan_traffic_uplink_preferences: list[
            UpdateNetworkApplianceTrafficShapingUplinkSelectionWanTrafficUplinkPreferencesItem
        ]
        | None = None,
        vpn_traffic_uplink_preferences: list[
            UpdateNetworkApplianceTrafficShapingUplinkSelectionVpnTrafficUplinkPreferencesItem
        ]
        | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update uplink selection settings for an MX network.

        [API documentation: updateNetworkApplianceTrafficShapingUplinkSelection](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-traffic-shaping-uplink-selection)

        Args:
            network_id: Network ID.
            active_active_auto_vpn_enabled: Toggle for enabling or disabling active-active AutoVPN.
            default_uplink: The default uplink. Must be a WAN interface 'wanX'.
            load_balancing_enabled: Toggle for enabling or disabling load balancing.
            failover_and_failback: WAN failover and failback behavior.
            wan_traffic_uplink_preferences: Array of uplink preference rules for WAN traffic.
            vpn_traffic_uplink_preferences: Array of uplink preference rules for VPN traffic.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/trafficShaping/uplinkSelection"

        payload: dict[str, Any] = {}
        if active_active_auto_vpn_enabled is not None:
            payload["activeActiveAutoVpnEnabled"] = active_active_auto_vpn_enabled
        if default_uplink is not None:
            payload["defaultUplink"] = default_uplink
        if load_balancing_enabled is not None:
            payload["loadBalancingEnabled"] = load_balancing_enabled
        if failover_and_failback is not None:
            payload["failoverAndFailback"] = failover_and_failback.model_dump(
                by_alias=True, exclude_none=True
            )
        if wan_traffic_uplink_preferences is not None:
            payload["wanTrafficUplinkPreferences"] = [
                item.model_dump(by_alias=True, exclude_none=True)
                for item in wan_traffic_uplink_preferences
            ]
        if vpn_traffic_uplink_preferences is not None:
            payload["vpnTrafficUplinkPreferences"] = [
                item.model_dump(by_alias=True, exclude_none=True)
                for item in vpn_traffic_uplink_preferences
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_traffic_shaping_vpn_exclusions(
        self,
        network_id: str,
        *,
        custom: list[UpdateNetworkApplianceTrafficShapingVpnExclusionsCustomItem] | None = None,
        major_applications: list[
            UpdateNetworkApplianceTrafficShapingVpnExclusionsMajorApplicationsItem
        ]
        | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update VPN exclusion rules for an MX network.

        [API documentation: updateNetworkApplianceTrafficShapingVpnExclusions](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-traffic-shaping-vpn-exclusions)

        Args:
            network_id: Network ID.
            custom: Custom VPN exclusion rules. Pass an empty array to clear existing rules.
            major_applications: Major Application based VPN exclusion rules. Pass an empty array to
                clear existing rules.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/trafficShaping/vpnExclusions"

        payload: dict[str, Any] = {}
        if custom is not None:
            payload["custom"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in custom
            ]
        if major_applications is not None:
            payload["majorApplications"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in major_applications
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def create_network_appliance_vlan(
        self,
        *,
        network_id: str,
        id: str,
        name: str,
        subnet: str | None = None,
        appliance_ip: str | None = None,
        group_policy_id: str | None = None,
        template_vlan_type: CreateNetworkApplianceVlanTemplateVlanType | None = None,
        cidr: str | None = None,
        mask: int | None = None,
        ipv6: CreateNetworkApplianceVlanIpv6 | None = None,
        dhcp_handling: CreateNetworkApplianceVlanDhcpHandling | None = None,
        dhcp_relay_server_ips: list[str] | None = None,
        dhcp_lease_time: CreateNetworkApplianceVlanDhcpLeaseTime | None = None,
        mandatory_dhcp: CreateNetworkApplianceVlanMandatoryDhcp | None = None,
        dhcp_boot_options_enabled: bool | None = None,
        dhcp_boot_next_server: str | None = None,
        dhcp_boot_filename: str | None = None,
        dhcp_options: list[CreateNetworkApplianceVlanDhcpOptionsItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Add a VLAN.

        [API documentation: createNetworkApplianceVlan](https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-vlan)

        Args:
            network_id: Network ID.
            id: The VLAN ID of the new VLAN (must be between 1 and 4094).
            name: The name of the new VLAN.
            subnet: The subnet of the VLAN.
            appliance_ip: The local IP of the appliance on the VLAN.
            group_policy_id: The id of the desired group policy to apply to the VLAN.
            template_vlan_type: Type of subnetting of the VLAN. Applicable only for template
                network.
            cidr: CIDR of the pool of subnets. Applicable only for template network. Each network
                bound to the template will automatically pick a subnet from this pool to
                build its own VLAN.
            mask: Mask used for the subnet of all bound to the template networks. Applicable only
                for template network.
            ipv6: IPv6 configuration on the VLAN.
            dhcp_handling: The appliance's handling of DHCP requests on this VLAN. One of: 'Run a
                DHCP server', 'Relay DHCP to another server' or 'Do not respond to DHCP
                requests'.
            dhcp_relay_server_ips: The IPs (IPv4) of the DHCP servers that DHCP requests should be
                relayed to. CIDR/subnet notation and hostnames are not supported.
            dhcp_lease_time: The term of DHCP leases if the appliance is running a DHCP server on
                this VLAN. One of: '30 minutes', '1 hour', '4 hours', '12 hours', '1
                day' or '1 week'.
            mandatory_dhcp: Mandatory DHCP will enforce that clients connecting to this VLAN must
                use the IP address assigned by the DHCP server. Clients who use a static
                IP address won't be able to associate. Only available on firmware
                versions 17.0 and above.
            dhcp_boot_options_enabled: Use DHCP boot options specified in other properties.
            dhcp_boot_next_server: DHCP boot option to direct boot clients to the server to load the
                boot file from.
            dhcp_boot_filename: DHCP boot option for boot filename.
            dhcp_options: The list of DHCP options that will be included in DHCP responses. Each
                object in the list should have "code", "type", and "value" properties.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/vlans"

        payload: dict[str, Any] = {}
        if id is not None:
            payload["id"] = id
        if name is not None:
            payload["name"] = name
        if subnet is not None:
            payload["subnet"] = subnet
        if appliance_ip is not None:
            payload["applianceIp"] = appliance_ip
        if group_policy_id is not None:
            payload["groupPolicyId"] = group_policy_id
        if template_vlan_type is not None:
            payload["templateVlanType"] = template_vlan_type
        if cidr is not None:
            payload["cidr"] = cidr
        if mask is not None:
            payload["mask"] = mask
        if ipv6 is not None:
            payload["ipv6"] = ipv6.model_dump(by_alias=True, exclude_none=True)
        if dhcp_handling is not None:
            payload["dhcpHandling"] = dhcp_handling
        if dhcp_relay_server_ips is not None:
            payload["dhcpRelayServerIps"] = dhcp_relay_server_ips
        if dhcp_lease_time is not None:
            payload["dhcpLeaseTime"] = dhcp_lease_time
        if mandatory_dhcp is not None:
            payload["mandatoryDhcp"] = mandatory_dhcp.model_dump(by_alias=True, exclude_none=True)
        if dhcp_boot_options_enabled is not None:
            payload["dhcpBootOptionsEnabled"] = dhcp_boot_options_enabled
        if dhcp_boot_next_server is not None:
            payload["dhcpBootNextServer"] = dhcp_boot_next_server
        if dhcp_boot_filename is not None:
            payload["dhcpBootFilename"] = dhcp_boot_filename
        if dhcp_options is not None:
            payload["dhcpOptions"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in dhcp_options
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_network_appliance_vlans_settings(
        self, network_id: str, *, vlans_enabled: bool | None = None
    ) -> CreateOrganizationActionBatchActionsItem:
        """Enable/Disable VLANs for the given network.

        [API documentation: updateNetworkApplianceVlansSettings](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-vlans-settings)

        Args:
            network_id: Network ID.
            vlans_enabled: Boolean indicating whether to enable (true) or disable (false) VLANs for
                the network.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/vlans/settings"

        payload: dict[str, Any] = {}
        if vlans_enabled is not None:
            payload["vlansEnabled"] = vlans_enabled

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_vlan(
        self,
        *,
        network_id: str,
        vlan_id: str,
        name: str | None = None,
        subnet: str | None = None,
        appliance_ip: str | None = None,
        group_policy_id: str | None = None,
        vpn_nat_subnet: str | None = None,
        dhcp_handling: UpdateNetworkApplianceVlanDhcpHandling | None = None,
        dhcp_relay_server_ips: list[str] | None = None,
        dhcp_lease_time: UpdateNetworkApplianceVlanDhcpLeaseTime | None = None,
        dhcp_boot_options_enabled: bool | None = None,
        dhcp_boot_next_server: str | None = None,
        dhcp_boot_filename: str | None = None,
        fixed_ip_assignments: dict[str, Any] | None = None,
        reserved_ip_ranges: list[UpdateNetworkApplianceVlanReservedIpRangesItem] | None = None,
        dns_nameservers: str | None = None,
        dhcp_options: list[UpdateNetworkApplianceVlanDhcpOptionsItem] | None = None,
        template_vlan_type: UpdateNetworkApplianceVlanTemplateVlanType | None = None,
        cidr: str | None = None,
        mask: int | None = None,
        ipv6: UpdateNetworkApplianceVlanIpv6 | None = None,
        mandatory_dhcp: UpdateNetworkApplianceVlanMandatoryDhcp | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a VLAN.

        [API documentation: updateNetworkApplianceVlan](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-vlan)

        Args:
            network_id: Network ID.
            vlan_id: Vlan ID.
            name: The name of the VLAN.
            subnet: The subnet of the VLAN.
            appliance_ip: The local IP of the appliance on the VLAN.
            group_policy_id: The id of the desired group policy to apply to the VLAN.
            vpn_nat_subnet: The translated VPN subnet if VPN and VPN subnet translation are enabled
                on the VLAN.
            dhcp_handling: The appliance's handling of DHCP requests on this VLAN. One of: 'Run a
                DHCP server', 'Relay DHCP to another server' or 'Do not respond to DHCP
                requests'.
            dhcp_relay_server_ips: The IPs (IPv4) of the DHCP servers that DHCP requests should be
                relayed to. CIDR/subnet notation and hostnames are not supported.
            dhcp_lease_time: The term of DHCP leases if the appliance is running a DHCP server on
                this VLAN. One of: '30 minutes', '1 hour', '4 hours', '12 hours', '1
                day' or '1 week'.
            dhcp_boot_options_enabled: Use DHCP boot options specified in other properties.
            dhcp_boot_next_server: DHCP boot option to direct boot clients to the server to load the
                boot file from.
            dhcp_boot_filename: DHCP boot option for boot filename.
            fixed_ip_assignments: The DHCP fixed IP assignments on the VLAN. This should be an
                object that contains mappings from MAC addresses to objects that
                themselves each contain "ip" and "name" string fields. See the sample
                request/response for more details.
            reserved_ip_ranges: The DHCP reserved IP ranges on the VLAN.
            dns_nameservers: The DNS nameservers used for DHCP responses, either "upstream_dns",
                "google_dns", "opendns", or a newline seperated string of IP addresses
                or domain names.
            dhcp_options: The list of DHCP options that will be included in DHCP responses. Each
                object in the list should have "code", "type", and "value" properties.
            template_vlan_type: Type of subnetting of the VLAN. Applicable only for template
                network.
            cidr: CIDR of the pool of subnets. Applicable only for template network. Each network
                bound to the template will automatically pick a subnet from this pool to
                build its own VLAN.
            mask: Mask used for the subnet of all bound to the template networks. Applicable only
                for template network.
            ipv6: IPv6 configuration on the VLAN.
            mandatory_dhcp: Mandatory DHCP will enforce that clients connecting to this VLAN must
                use the IP address assigned by the DHCP server. Clients who use a static
                IP address won't be able to associate. Only available on firmware
                versions 17.0 and above.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        vlan_id = urllib.parse.quote(str(vlan_id), safe="")
        path = f"/networks/{network_id}/appliance/vlans/{vlan_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if subnet is not None:
            payload["subnet"] = subnet
        if appliance_ip is not None:
            payload["applianceIp"] = appliance_ip
        if group_policy_id is not None:
            payload["groupPolicyId"] = group_policy_id
        if vpn_nat_subnet is not None:
            payload["vpnNatSubnet"] = vpn_nat_subnet
        if dhcp_handling is not None:
            payload["dhcpHandling"] = dhcp_handling
        if dhcp_relay_server_ips is not None:
            payload["dhcpRelayServerIps"] = dhcp_relay_server_ips
        if dhcp_lease_time is not None:
            payload["dhcpLeaseTime"] = dhcp_lease_time
        if dhcp_boot_options_enabled is not None:
            payload["dhcpBootOptionsEnabled"] = dhcp_boot_options_enabled
        if dhcp_boot_next_server is not None:
            payload["dhcpBootNextServer"] = dhcp_boot_next_server
        if dhcp_boot_filename is not None:
            payload["dhcpBootFilename"] = dhcp_boot_filename
        if fixed_ip_assignments is not None:
            payload["fixedIpAssignments"] = fixed_ip_assignments
        if reserved_ip_ranges is not None:
            payload["reservedIpRanges"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in reserved_ip_ranges
            ]
        if dns_nameservers is not None:
            payload["dnsNameservers"] = dns_nameservers
        if dhcp_options is not None:
            payload["dhcpOptions"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in dhcp_options
            ]
        if template_vlan_type is not None:
            payload["templateVlanType"] = template_vlan_type
        if cidr is not None:
            payload["cidr"] = cidr
        if mask is not None:
            payload["mask"] = mask
        if ipv6 is not None:
            payload["ipv6"] = ipv6.model_dump(by_alias=True, exclude_none=True)
        if mandatory_dhcp is not None:
            payload["mandatoryDhcp"] = mandatory_dhcp.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_network_appliance_vlan(
        self, *, network_id: str, vlan_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Delete a VLAN from a network.

        [API documentation: deleteNetworkApplianceVlan](https://developer.cisco.com/meraki/api-v1/#!delete-network-appliance-vlan)

        Args:
            network_id: Network ID.
            vlan_id: Vlan ID.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        vlan_id = urllib.parse.quote(str(vlan_id), safe="")
        path = f"/networks/{network_id}/appliance/vlans/{vlan_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def update_network_appliance_vpn_bgp(
        self,
        *,
        network_id: str,
        enabled: bool,
        as_number: int | None = None,
        ibgp_hold_timer: int | None = None,
        neighbors: list[UpdateNetworkApplianceVpnBgpNeighborsItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a Hub BGP Configuration.

        [API documentation: updateNetworkApplianceVpnBgp](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-vpn-bgp)

        Args:
            network_id: Network ID.
            enabled: Boolean value to enable or disable the BGP configuration. When BGP is enabled,
                the asNumber (ASN) will be autopopulated with the preconfigured ASN at
                other Hubs or a default value if there is no ASN configured.
            as_number: An Autonomous System Number (ASN) is required if you are to run BGP and peer
                with another BGP Speaker outside of the Auto VPN domain. This ASN will
                be applied to the entire Auto VPN domain. The entire 4-byte ASN range is
                supported. So, the ASN must be an integer between 1 and 4294967295. When
                absent, this field is not updated. If no value exists then it defaults
                to 64512.
            ibgp_hold_timer: The iBGP holdtimer in seconds. The iBGP holdtimer must be an integer
                between 12 and 240. When absent, this field is not updated. If no value
                exists then it defaults to 240.
            neighbors: List of BGP neighbors. This list replaces the existing set of neighbors. When
                absent, this field is not updated.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/vpn/bgp"

        payload: dict[str, Any] = {}
        if enabled is not None:
            payload["enabled"] = enabled
        if as_number is not None:
            payload["asNumber"] = as_number
        if ibgp_hold_timer is not None:
            payload["ibgpHoldTimer"] = ibgp_hold_timer
        if neighbors is not None:
            payload["neighbors"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in neighbors
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="settings/update",
            body=payload,
        )

    def update_network_appliance_vpn_site_to_site_vpn(
        self,
        *,
        network_id: str,
        mode: UpdateNetworkApplianceVpnSiteToSiteVpnMode,
        hubs: list[UpdateNetworkApplianceVpnSiteToSiteVpnHubsItem] | None = None,
        subnets: list[UpdateNetworkApplianceVpnSiteToSiteVpnSubnetsItem] | None = None,
        subnet: UpdateNetworkApplianceVpnSiteToSiteVpnSubnet | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the site-to-site VPN settings of a network.

        [API documentation: updateNetworkApplianceVpnSiteToSiteVpn](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-vpn-site-to-site-vpn)

        Args:
            network_id: Network ID.
            mode: The site-to-site VPN mode. Can be one of 'none', 'spoke' or 'hub'.
            hubs: The list of VPN hubs, in order of preference. In spoke mode, at least 1 hub is
                required.
            subnets: The list of subnets and their VPN presence.
            subnet: Configuration of subnet features.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/vpn/siteToSiteVpn"

        payload: dict[str, Any] = {}
        if mode is not None:
            payload["mode"] = mode
        if hubs is not None:
            payload["hubs"] = [item.model_dump(by_alias=True, exclude_none=True) for item in hubs]
        if subnets is not None:
            payload["subnets"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in subnets
            ]
        if subnet is not None:
            payload["subnet"] = subnet.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_network_appliance_warm_spare(
        self,
        *,
        network_id: str,
        enabled: bool,
        spare_serial: str | None = None,
        uplink_mode: str | None = None,
        virtual_ip1: str | None = None,
        virtual_ip2: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update MX warm spare settings.

        [API documentation: updateNetworkApplianceWarmSpare](https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-warm-spare)

        Args:
            network_id: Network ID.
            enabled: Enable warm spare.
            spare_serial: Serial number of the warm spare appliance.
            uplink_mode: Uplink mode, either virtual or public.
            virtual_ip1: The WAN 1 shared IP.
            virtual_ip2: The WAN 2 shared IP.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/warmSpare"

        payload: dict[str, Any] = {}
        if enabled is not None:
            payload["enabled"] = enabled
        if spare_serial is not None:
            payload["spareSerial"] = spare_serial
        if uplink_mode is not None:
            payload["uplinkMode"] = uplink_mode
        if virtual_ip1 is not None:
            payload["virtualIp1"] = virtual_ip1
        if virtual_ip2 is not None:
            payload["virtualIp2"] = virtual_ip2

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def swap_network_appliance_warm_spare(
        self, network_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Swap MX primary and warm spare appliances.

        [API documentation: swapNetworkApplianceWarmSpare](https://developer.cisco.com/meraki/api-v1/#!swap-network-appliance-warm-spare)

        Args:
            network_id: Network ID.

        """
        network_id = urllib.parse.quote(str(network_id), safe="")
        path = f"/networks/{network_id}/appliance/warmSpare/swap"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="swap",
        )

    def create_organization_appliance_dns_local_profile(
        self, *, organization_id: str, name: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a new local DNS profile.

        [API documentation: createOrganizationApplianceDnsLocalProfile](https://developer.cisco.com/meraki/api-v1/#!create-organization-appliance-dns-local-profile)

        Args:
            organization_id: Organization ID.
            name: Name of profile.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/local/profiles"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def bulk_organization_appliance_dns_local_profiles_assignments_create(
        self,
        *,
        organization_id: str,
        items: list[BulkOrganizationApplianceDnsLocalProfilesAssignmentsCreateItemsItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Assign the local DNS profile to networks in the organization.

        [API documentation: bulkOrganizationApplianceDnsLocalProfilesAssignmentsCreate](https://developer.cisco.com/meraki/api-v1/#!bulk-organization-appliance-dns-local-profiles-assignments-create)

        Args:
            organization_id: Organization ID.
            items: List containing the network ID and Profile ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = (
            f"/organizations/{organization_id}/appliance/dns/local/profiles/assignments/bulkCreate"
        )

        payload: dict[str, Any] = {}
        if items is not None:
            payload["items"] = [item.model_dump(by_alias=True, exclude_none=True) for item in items]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="bulk_create",
            body=payload,
        )

    def create_organization_appliance_dns_local_profiles_assignments_bulk_delete(
        self,
        *,
        organization_id: str,
        items: list[CreateOrganizationApplianceDnsLocalProfilesAssignmentsBulkDeleteItemsItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Unassign the local DNS profile to networks in the organization.

        [API documentation: createOrganizationApplianceDnsLocalProfilesAssignmentsBulkDelete](https://developer.cisco.com/meraki/api-v1/#!create-organization-appliance-dns-local-profiles-assignments-bulk-delete)

        Args:
            organization_id: Organization ID.
            items: List containing the assignment ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = (
            f"/organizations/{organization_id}/appliance/dns/local/profiles/assignments/bulkDelete"
        )

        payload: dict[str, Any] = {}
        if items is not None:
            payload["items"] = [item.model_dump(by_alias=True, exclude_none=True) for item in items]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="bulk_delete",
            body=payload,
        )

    def update_organization_appliance_dns_local_profile(
        self, *, organization_id: str, profile_id: str, name: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a local DNS profile.

        [API documentation: updateOrganizationApplianceDnsLocalProfile](https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-dns-local-profile)

        Args:
            organization_id: Organization ID.
            profile_id: Profile ID.
            name: Name of profile.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        profile_id = urllib.parse.quote(str(profile_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/local/profiles/{profile_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_appliance_dns_local_profile(
        self, *, organization_id: str, profile_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Deletes a local DNS profile.

        [API documentation: deleteOrganizationApplianceDnsLocalProfile](https://developer.cisco.com/meraki/api-v1/#!delete-organization-appliance-dns-local-profile)

        Args:
            organization_id: Organization ID.
            profile_id: Profile ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        profile_id = urllib.parse.quote(str(profile_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/local/profiles/{profile_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_appliance_dns_local_record(
        self,
        *,
        organization_id: str,
        hostname: str,
        address: str,
        profile: CreateOrganizationApplianceDnsLocalRecordProfile,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a new local DNS record.

        [API documentation: createOrganizationApplianceDnsLocalRecord](https://developer.cisco.com/meraki/api-v1/#!create-organization-appliance-dns-local-record)

        Args:
            organization_id: Organization ID.
            hostname: Hostname for the DNS record.
            address: IP for the DNS record.
            profile: The profile the DNS record is associated with.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/local/records"

        payload: dict[str, Any] = {}
        if hostname is not None:
            payload["hostname"] = hostname
        if address is not None:
            payload["address"] = address
        if profile is not None:
            payload["profile"] = profile.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def update_organization_appliance_dns_local_record(
        self,
        *,
        organization_id: str,
        record_id: str,
        hostname: str | None = None,
        address: str | None = None,
        profile: UpdateOrganizationApplianceDnsLocalRecordProfile | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Updates a local DNS record.

        [API documentation: updateOrganizationApplianceDnsLocalRecord](https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-dns-local-record)

        Args:
            organization_id: Organization ID.
            record_id: Record ID.
            hostname: Hostname for the DNS record.
            address: IP for the DNS record.
            profile: The profile the DNS record is associated with.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        record_id = urllib.parse.quote(str(record_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/local/records/{record_id}"

        payload: dict[str, Any] = {}
        if hostname is not None:
            payload["hostname"] = hostname
        if address is not None:
            payload["address"] = address
        if profile is not None:
            payload["profile"] = profile.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_appliance_dns_local_record(
        self, *, organization_id: str, record_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Deletes a local DNS record.

        [API documentation: deleteOrganizationApplianceDnsLocalRecord](https://developer.cisco.com/meraki/api-v1/#!delete-organization-appliance-dns-local-record)

        Args:
            organization_id: Organization ID.
            record_id: Record ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        record_id = urllib.parse.quote(str(record_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/local/records/{record_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def create_organization_appliance_dns_split_profile(
        self,
        *,
        organization_id: str,
        name: str,
        hostnames: list[str],
        nameservers: CreateOrganizationApplianceDnsSplitProfileNameservers,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Create a new split DNS profile.

        [API documentation: createOrganizationApplianceDnsSplitProfile](https://developer.cisco.com/meraki/api-v1/#!create-organization-appliance-dns-split-profile)

        Args:
            organization_id: Organization ID.
            name: Name of profile.
            hostnames: The hostname patterns to match for redirection. For more information on Split
                DNS hostname pattern formatting, please consult the Split DNS KB.
            nameservers: Contains the nameserver information for redirection.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/split/profiles"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if hostnames is not None:
            payload["hostnames"] = hostnames
        if nameservers is not None:
            payload["nameservers"] = nameservers.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="create",
            body=payload,
        )

    def create_organization_appliance_dns_split_profiles_assignments_bulk_create(
        self,
        *,
        organization_id: str,
        items: list[CreateOrganizationApplianceDnsSplitProfilesAssignmentsBulkCreateItemsItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Assign the split DNS profile to networks in the organization.

        [API documentation: createOrganizationApplianceDnsSplitProfilesAssignmentsBulkCreate](https://developer.cisco.com/meraki/api-v1/#!create-organization-appliance-dns-split-profiles-assignments-bulk-create)

        Args:
            organization_id: Organization ID.
            items: List containing the network ID and Profile ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = (
            f"/organizations/{organization_id}/appliance/dns/split/profiles/assignments/bulkCreate"
        )

        payload: dict[str, Any] = {}
        if items is not None:
            payload["items"] = [item.model_dump(by_alias=True, exclude_none=True) for item in items]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="bulk_create",
            body=payload,
        )

    def create_organization_appliance_dns_split_profiles_assignments_bulk_delete(
        self,
        *,
        organization_id: str,
        items: list[CreateOrganizationApplianceDnsSplitProfilesAssignmentsBulkDeleteItemsItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Unassign the split DNS profile to networks in the organization.

        [API documentation: createOrganizationApplianceDnsSplitProfilesAssignmentsBulkDelete](https://developer.cisco.com/meraki/api-v1/#!create-organization-appliance-dns-split-profiles-assignments-bulk-delete)

        Args:
            organization_id: Organization ID.
            items: List containing the assignment ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = (
            f"/organizations/{organization_id}/appliance/dns/split/profiles/assignments/bulkDelete"
        )

        payload: dict[str, Any] = {}
        if items is not None:
            payload["items"] = [item.model_dump(by_alias=True, exclude_none=True) for item in items]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="bulk_delete",
            body=payload,
        )

    def update_organization_appliance_dns_split_profile(
        self,
        *,
        organization_id: str,
        profile_id: str,
        name: str | None = None,
        hostnames: list[str] | None = None,
        nameservers: UpdateOrganizationApplianceDnsSplitProfileNameservers | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update a split DNS profile.

        [API documentation: updateOrganizationApplianceDnsSplitProfile](https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-dns-split-profile)

        Args:
            organization_id: Organization ID.
            profile_id: Profile ID.
            name: Name of profile.
            hostnames: The hostname patterns to match for redirection. For more information on Split
                DNS hostname pattern formatting, please consult the Split DNS KB.
            nameservers: Contains the nameserver information for redirection.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        profile_id = urllib.parse.quote(str(profile_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/split/profiles/{profile_id}"

        payload: dict[str, Any] = {}
        if name is not None:
            payload["name"] = name
        if hostnames is not None:
            payload["hostnames"] = hostnames
        if nameservers is not None:
            payload["nameservers"] = nameservers.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def delete_organization_appliance_dns_split_profile(
        self, *, organization_id: str, profile_id: str
    ) -> CreateOrganizationActionBatchActionsItem:
        """Deletes a split DNS profile.

        [API documentation: deleteOrganizationApplianceDnsSplitProfile](https://developer.cisco.com/meraki/api-v1/#!delete-organization-appliance-dns-split-profile)

        Args:
            organization_id: Organization ID.
            profile_id: Profile ID.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        profile_id = urllib.parse.quote(str(profile_id), safe="")
        path = f"/organizations/{organization_id}/appliance/dns/split/profiles/{profile_id}"

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="destroy",
        )

    def update_organization_appliance_vpn_site_to_site_ipsec_peers_slas(
        self,
        organization_id: str,
        *,
        items: list[UpdateOrganizationApplianceVpnSiteToSiteIpsecPeersSlasItemsItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the IPsec SLA policies for an organization.

        [API documentation: updateOrganizationApplianceVpnSiteToSiteIpsecPeersSlas](https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-vpn-site-to-site-ipsec-peers-slas)

        Args:
            organization_id: Organization ID.
            items: List of IPsec SLA policies.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/appliance/vpn/siteToSite/ipsec/peers/slas"

        payload: dict[str, Any] = {}
        if items is not None:
            payload["items"] = [item.model_dump(by_alias=True, exclude_none=True) for item in items]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_organization_appliance_vpn_third_party_vpn_peers(
        self,
        *,
        organization_id: str,
        peers: list[UpdateOrganizationApplianceVpnThirdPartyVPNPeersPeersItem],
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update the third party VPN peers for an organization.

        [API documentation: updateOrganizationApplianceVpnThirdPartyVPNPeers](https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-vpn-third-party-vpn-peers)

        Args:
            organization_id: Organization ID.
            peers: The list of VPN peers.

        """
        organization_id = urllib.parse.quote(str(organization_id), safe="")
        path = f"/organizations/{organization_id}/appliance/vpn/thirdPartyVPNPeers"

        payload: dict[str, Any] = {}
        if peers is not None:
            payload["peers"] = [item.model_dump(by_alias=True, exclude_none=True) for item in peers]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )



"""Logging setup for codegen."""

import logging
from typing import ClassVar


class ColorFormatter(logging.Formatter):
    """Color formatter for logging."""

    COLORS: ClassVar[dict[int, str]] = {
        logging.DEBUG: "\033[36m",  # Cyan
        logging.INFO: "\033[32m",  # Green
        logging.WARNING: "\033[33m",  # Yellow
        logging.ERROR: "\033[31m",  # Red
        logging.CRITICAL: "\033[35m",  # Magenta
    }
    RESET = "\033[0m"

    def format(self, record: logging.LogRecord) -> str:
        """Format a log record."""
        color = self.COLORS.get(record.levelno, "")
        record.levelname = f"{color}{record.levelname}{self.RESET}"
        return super().format(record)


def setup_logging(level: int = logging.INFO) -> None:
    """Setup logging."""
    handler = logging.StreamHandler()
    handler.setFormatter(ColorFormatter("%(levelname)s - %(message)s"))
    logging.root.addHandler(handler)
    logging.root.setLevel(level)



"""Tests for cellularGateway API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from meraki_client import MerakiClient
from tests.generated.conftest import skip_on_unsupported


def test_get_device_cellular_gateway_lan(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_cellular_gateway_lan endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_device_cellular_gateway_lan(serial=device_serial)


def test_get_device_cellular_gateway_port_forwarding_rules(
    client: MerakiClient, device_serial: str
) -> None:
    """Test get_device_cellular_gateway_port_forwarding_rules endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_device_cellular_gateway_port_forwarding_rules(
            serial=device_serial
        )


def test_get_network_cellular_gateway_connectivity_monitoring_destinations(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_cellular_gateway_connectivity_monitoring_destinations endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_network_cellular_gateway_connectivity_monitoring_destinations(
            network_id=network_id
        )


def test_get_network_cellular_gateway_dhcp(client: MerakiClient, network_id: str) -> None:
    """Test get_network_cellular_gateway_dhcp endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_network_cellular_gateway_dhcp(network_id=network_id)


def test_get_network_cellular_gateway_subnet_pool(client: MerakiClient, network_id: str) -> None:
    """Test get_network_cellular_gateway_subnet_pool endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_network_cellular_gateway_subnet_pool(network_id=network_id)


def test_get_network_cellular_gateway_uplink(client: MerakiClient, network_id: str) -> None:
    """Test get_network_cellular_gateway_uplink endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_network_cellular_gateway_uplink(network_id=network_id)


def test_get_organization_cellular_gateway_esims_inventory(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_cellular_gateway_esims_inventory endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_organization_cellular_gateway_esims_inventory(
            organization_id=organization_id
        )


def test_get_organization_cellular_gateway_esims_service_providers(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_cellular_gateway_esims_service_providers endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_organization_cellular_gateway_esims_service_providers(
            organization_id=organization_id
        )


def test_get_organization_cellular_gateway_esims_service_providers_accounts(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_cellular_gateway_esims_service_providers_accounts endpoint."""
    with skip_on_unsupported():
        client.cellular_gateway.get_organization_cellular_gateway_esims_service_providers_accounts(
            organization_id=organization_id
        )


def test_get_organization_cellular_gateway_uplink_statuses(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_cellular_gateway_uplink_statuses endpoint."""
    with skip_on_unsupported():
        result = client.cellular_gateway.get_organization_cellular_gateway_uplink_statuses(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)



"""Response schemas for sensor endpoints.

This module is automatically @generated from the Meraki API specification.
Do not edit this file manually.
"""

from __future__ import annotations

from datetime import datetime

from pydantic import Field, RootModel

from meraki_client.schemas._base import _BaseSchema


class CreateDeviceSensorCommandResponse(_BaseSchema):
    """Response for createDeviceSensorCommand operation."""

    command_id: str | None = Field(
        default=None, validation_alias="commandId", serialization_alias="commandId"
    )
    created_at: datetime | None = Field(
        default=None, validation_alias="createdAt", serialization_alias="createdAt"
    )
    completed_at: datetime | None = Field(
        default=None, validation_alias="completedAt", serialization_alias="completedAt"
    )
    created_by: SensorCreatedBy | None = Field(
        default=None, validation_alias="createdBy", serialization_alias="createdBy"
    )
    operation: str | None = None
    status: str | None = None
    errors: list[str] = Field(default_factory=list)


class CreateNetworkSensorAlertsProfileConditionsItem(_BaseSchema):
    """Item schema for conditions."""

    metric: str
    threshold: SensorThreshold
    direction: str | None = None
    duration: int | None = None


class CreateNetworkSensorAlertsProfileRecipients(_BaseSchema):
    """List of recipients that will receive the alert."""

    emails: list[str] = Field(default_factory=list)
    sms_numbers: list[str] = Field(
        default_factory=list, validation_alias="smsNumbers", serialization_alias="smsNumbers"
    )
    http_server_ids: list[str] = Field(
        default_factory=list, validation_alias="httpServerIds", serialization_alias="httpServerIds"
    )


class CreateNetworkSensorAlertsProfileResponse(_BaseSchema):
    """Response for createNetworkSensorAlertsProfile operation."""

    profile_id: str | None = Field(
        default=None, validation_alias="profileId", serialization_alias="profileId"
    )
    name: str | None = None
    schedule: SensorSchedule | None = None
    conditions: list[SensorConditionsItem]
    recipients: SensorRecipients | None = None
    serials: list[str] = Field(default_factory=list)
    include_sensor_url: bool | None = Field(
        default=None, validation_alias="includeSensorUrl", serialization_alias="includeSensorUrl"
    )
    message: str | None = None


class CreateNetworkSensorAlertsProfileSchedule(_BaseSchema):
    """The sensor schedule to use with the alert profile."""

    id: str | None = None


class GetDeviceSensorCommandResponse(_BaseSchema):
    """Response for getDeviceSensorCommand operation."""

    command_id: str | None = Field(
        default=None, validation_alias="commandId", serialization_alias="commandId"
    )
    created_at: datetime | None = Field(
        default=None, validation_alias="createdAt", serialization_alias="createdAt"
    )
    completed_at: datetime | None = Field(
        default=None, validation_alias="completedAt", serialization_alias="completedAt"
    )
    created_by: SensorCreatedBy | None = Field(
        default=None, validation_alias="createdBy", serialization_alias="createdBy"
    )
    operation: str | None = None
    status: str | None = None
    errors: list[str] = Field(default_factory=list)


class GetDeviceSensorCommandsResponse(RootModel[list["GetDeviceSensorCommandsResponseItem"]]):
    """Response for getDeviceSensorCommands operation."""


class GetDeviceSensorCommandsResponseItem(_BaseSchema):
    """Schema for GetDeviceSensorCommandsResponseItem."""

    command_id: str | None = Field(
        default=None, validation_alias="commandId", serialization_alias="commandId"
    )
    created_at: datetime | None = Field(
        default=None, validation_alias="createdAt", serialization_alias="createdAt"
    )
    completed_at: datetime | None = Field(
        default=None, validation_alias="completedAt", serialization_alias="completedAt"
    )
    created_by: SensorCreatedBy | None = Field(
        default=None, validation_alias="createdBy", serialization_alias="createdBy"
    )
    operation: str | None = None
    status: str | None = None
    errors: list[str] = Field(default_factory=list)


class GetDeviceSensorRelationshipsResponse(_BaseSchema):
    """Response for getDeviceSensorRelationships operation."""

    livestream: GetDeviceSensorRelationshipsResponseLivestream | None = None


class GetDeviceSensorRelationshipsResponseLivestream(_BaseSchema):
    """A role defined between an MT sensor and an MV camera that adds the camera's livestream to
    the sensor's details page. Snapshots from the camera will also appear in alert notifications
    that the sensor triggers.
    """

    related_devices: list[SensorLivestreamRelatedDevicesItem] = Field(
        default_factory=list,
        validation_alias="relatedDevices",
        serialization_alias="relatedDevices",
    )


class GetNetworkSensorAlertsCurrentOverviewByMetricResponse(_BaseSchema):
    """Response for getNetworkSensorAlertsCurrentOverviewByMetric operation."""

    supported_metrics: list[str] = Field(
        default_factory=list,
        validation_alias="supportedMetrics",
        serialization_alias="supportedMetrics",
    )
    counts: GetNetworkSensorAlertsCurrentOverviewByMetricResponseCounts | None = None


class GetNetworkSensorAlertsCurrentOverviewByMetricResponseCounts(_BaseSchema):
    """Counts of currently alerting sensors, aggregated by alerting metric."""

    apparent_power: int | None = Field(
        default=None, validation_alias="apparentPower", serialization_alias="apparentPower"
    )
    co2: int | None = None
    current: int | None = None
    door: int | None = None
    frequency: int | None = None
    humidity: int | None = None
    indoor_air_quality: int | None = Field(
        default=None, validation_alias="indoorAirQuality", serialization_alias="indoorAirQuality"
    )
    noise: SensorCountsNoise | None = None
    pm25: int | None = None
    power_factor: int | None = Field(
        default=None, validation_alias="powerFactor", serialization_alias="powerFactor"
    )
    real_power: int | None = Field(
        default=None, validation_alias="realPower", serialization_alias="realPower"
    )
    temperature: int | None = None
    tvoc: int | None = None
    upstream_power: int | None = Field(
        default=None, validation_alias="upstreamPower", serialization_alias="upstreamPower"
    )
    voltage: int | None = None
    water: int | None = None


class GetNetworkSensorAlertsOverviewByMetricResponse(
    RootModel[list["GetNetworkSensorAlertsOverviewByMetricResponseItem"]]
):
    """Response for getNetworkSensorAlertsOverviewByMetric operation."""


class GetNetworkSensorAlertsOverviewByMetricResponseItem(_BaseSchema):
    """Schema for GetNetworkSensorAlertsOverviewByMetricResponseItem."""

    start_ts: datetime | None = Field(
        default=None, validation_alias="startTs", serialization_alias="startTs"
    )
    end_ts: datetime | None = Field(
        default=None, validation_alias="endTs", serialization_alias="endTs"
    )
    counts: GetNetworkSensorAlertsCurrentOverviewByMetricResponseCounts | None = None


class GetNetworkSensorAlertsProfileResponse(_BaseSchema):
    """Response for getNetworkSensorAlertsProfile operation."""

    profile_id: str | None = Field(
        default=None, validation_alias="profileId", serialization_alias="profileId"
    )
    name: str | None = None
    schedule: SensorSchedule | None = None
    conditions: list[SensorConditionsItem]
    recipients: SensorRecipients | None = None
    serials: list[str] = Field(default_factory=list)
    include_sensor_url: bool | None = Field(
        default=None, validation_alias="includeSensorUrl", serialization_alias="includeSensorUrl"
    )
    message: str | None = None


class GetNetworkSensorAlertsProfilesResponse(
    RootModel[list["GetNetworkSensorAlertsProfilesResponseItem"]]
):
    """Response for getNetworkSensorAlertsProfiles operation."""


class GetNetworkSensorAlertsProfilesResponseItem(_BaseSchema):
    """Schema for GetNetworkSensorAlertsProfilesResponseItem."""

    profile_id: str | None = Field(
        default=None, validation_alias="profileId", serialization_alias="profileId"
    )
    name: str | None = None
    schedule: SensorSchedule | None = None
    conditions: list[SensorConditionsItem]
    recipients: SensorRecipients | None = None
    serials: list[str] = Field(default_factory=list)
    include_sensor_url: bool | None = Field(
        default=None, validation_alias="includeSensorUrl", serialization_alias="includeSensorUrl"
    )
    message: str | None = None


class GetNetworkSensorMqttBrokerResponse(_BaseSchema):
    """Response for getNetworkSensorMqttBroker operation."""

    mqtt_broker_id: str | None = Field(
        default=None, validation_alias="mqttBrokerId", serialization_alias="mqttBrokerId"
    )
    enabled: bool | None = None


class GetNetworkSensorMqttBrokersResponse(
    RootModel[list["GetNetworkSensorMqttBrokersResponseItem"]]
):
    """Response for getNetworkSensorMqttBrokers operation."""


class GetNetworkSensorMqttBrokersResponseItem(_BaseSchema):
    """Schema for GetNetworkSensorMqttBrokersResponseItem."""

    mqtt_broker_id: str | None = Field(
        default=None, validation_alias="mqttBrokerId", serialization_alias="mqttBrokerId"
    )
    enabled: bool | None = None


class GetNetworkSensorRelationshipsResponse(
    RootModel[list["GetNetworkSensorRelationshipsResponseItem"]]
):
    """Response for getNetworkSensorRelationships operation."""


class GetNetworkSensorRelationshipsResponseItem(_BaseSchema):
    """Schema for GetNetworkSensorRelationshipsResponseItem."""

    device: SensorDevice | None = None
    relationships: SensorRelationships | None = None


class GetOrganizationSensorGatewaysConnectionsLatestResponseItemsItem(_BaseSchema):
    """Schema for GetOrganizationSensorGatewaysConnectionsLatestResponseItemsItem."""

    last_reported_at: datetime = Field(
        validation_alias="lastReportedAt", serialization_alias="lastReportedAt"
    )
    last_connected_at: datetime | None = Field(
        default=None, validation_alias="lastConnectedAt", serialization_alias="lastConnectedAt"
    )
    rssi: int
    network: SensorNetwork
    sensor: SensorSensor
    gateway: SensorSensor


class GetOrganizationSensorReadingsHistoryResponse(
    RootModel[list["GetOrganizationSensorReadingsHistoryResponseItem"]]
):
    """Response for getOrganizationSensorReadingsHistory operation."""


class GetOrganizationSensorReadingsHistoryResponseItem(_BaseSchema):
    """Schema for GetOrganizationSensorReadingsHistoryResponseItem."""

    serial: str | None = None
    network: SensorSchedule | None = None
    ts: str | None = None
    metric: str | None = None
    apparent_power: SensorApparentPower | None = Field(
        default=None, validation_alias="apparentPower", serialization_alias="apparentPower"
    )
    battery: SensorBattery | None = None
    button: SensorButton | None = None
    co2: SensorCo2 | None = None
    current: SensorApparentPower | None = None
    door: SensorDoor2 | None = None
    downstream_power: SensorDownstreamPower | None = Field(
        default=None, validation_alias="downstreamPower", serialization_alias="downstreamPower"
    )
    frequency: SensorFrequency | None = None
    humidity: SensorHumidity2 | None = None
    indoor_air_quality: SensorIndoorAirQuality2 | None = Field(
        default=None, validation_alias="indoorAirQuality", serialization_alias="indoorAirQuality"
    )
    noise: SensorNoise2 | None = None
    no2: SensorCo2 | None = None
    o3: SensorCo2 | None = None
    pm10: SensorCo2 | None = None
    pm25: SensorCo2 | None = None
    power_factor: SensorBattery | None = Field(
        default=None, validation_alias="powerFactor", serialization_alias="powerFactor"
    )
    real_power: SensorApparentPower | None = Field(
        default=None, validation_alias="realPower", serialization_alias="realPower"
    )
    remote_lockout_switch: SensorRemoteLockoutSwitch | None = Field(
        default=None,
        validation_alias="remoteLockoutSwitch",
        serialization_alias="remoteLockoutSwitch",
    )
    temperature: SensorTemperature2 | None = None
    tvoc: SensorCo2 | None = None
    voltage: SensorFrequency | None = None
    water: SensorWater2 | None = None
    raw_temperature: SensorTemperature2 | None = Field(
        default=None, validation_alias="rawTemperature", serialization_alias="rawTemperature"
    )


class GetOrganizationSensorReadingsLatestResponse(
    RootModel[list["GetOrganizationSensorReadingsLatestResponseItem"]]
):
    """Response for getOrganizationSensorReadingsLatest operation."""


class GetOrganizationSensorReadingsLatestResponseItem(_BaseSchema):
    """Schema for GetOrganizationSensorReadingsLatestResponseItem."""

    serial: str | None = None
    network: SensorSchedule | None = None
    readings: list[SensorReadingsItem] = Field(default_factory=list)


class SensorAmbient(_BaseSchema):
    """Ambient noise threshold. One of 'level' or 'quality' must be provided."""

    level: int | None = None
    quality: str | None = None


class SensorAmbient2(_BaseSchema):
    """Ambient noise reading."""

    level: int | None = None


class SensorApparentPower(_BaseSchema):
    """Reading for the 'apparentPower' metric. This will only be present if the 'metric' property
    equals 'apparentPower'.
    """

    draw: float | None = None


class SensorBattery(_BaseSchema):
    """Reading for the 'battery' metric. This will only be present if the 'metric' property equals
    'battery'.
    """

    percentage: int | None = None


class SensorButton(_BaseSchema):
    """Reading for the 'button' metric. This will only be present if the 'metric' property equals
    'button'.
    """

    press_type: str | None = Field(
        default=None, validation_alias="pressType", serialization_alias="pressType"
    )


class SensorCo2(_BaseSchema):
    """Reading for the 'co2' metric. This will only be present if the 'metric' property equals
    'co2'.
    """

    concentration: int | None = None


class SensorConditionsItem(_BaseSchema):
    """Schema for SensorConditionsItem."""

    metric: str
    threshold: SensorThreshold
    direction: str | None = None
    duration: int | None = None


class SensorCountsNoise(_BaseSchema):
    """Object containing the number of sensors that are currently alerting due to noise readings."""

    ambient: int | None = None


class SensorCreatedBy(_BaseSchema):
    """Information about the admin who triggered the command."""

    admin_id: str | None = Field(
        default=None, validation_alias="adminId", serialization_alias="adminId"
    )
    name: str | None = None
    email: str | None = None


class SensorDevice(_BaseSchema):
    """A sensor or gateway device in the network."""

    name: str | None = None
    serial: str | None = None
    product_type: str | None = Field(
        default=None, validation_alias="productType", serialization_alias="productType"
    )


class SensorDoor(_BaseSchema):
    """Door open threshold. 'open' must be provided and set to true."""

    open_: bool = Field(validation_alias="open", serialization_alias="open")


class SensorDoor2(_BaseSchema):
    """Reading for the 'door' metric. This will only be present if the 'metric' property equals
    'door'.
    """

    open_: bool | None = Field(default=None, validation_alias="open", serialization_alias="open")


class SensorDownstreamPower(_BaseSchema):
    """Reading for the 'downstreamPower' metric. This will only be present if the 'metric' property
    equals 'downstreamPower'.
    """

    enabled: bool | None = None


class SensorFrequency(_BaseSchema):
    """Reading for the 'frequency' metric. This will only be present if the 'metric' property
    equals 'frequency'.
    """

    level: float | None = None


class SensorHumidity(_BaseSchema):
    """Humidity threshold. One of 'relativePercentage' or 'quality' must be provided."""

    relative_percentage: int | None = Field(
        default=None,
        validation_alias="relativePercentage",
        serialization_alias="relativePercentage",
    )
    quality: str | None = None


class SensorHumidity2(_BaseSchema):
    """Reading for the 'humidity' metric. This will only be present if the 'metric' property equals
    'humidity'.
    """

    relative_percentage: int | None = Field(
        default=None,
        validation_alias="relativePercentage",
        serialization_alias="relativePercentage",
    )


class SensorIndoorAirQuality(_BaseSchema):
    """Indoor air quality score threshold. One of 'score' or 'quality' must be provided."""

    score: int | None = None
    quality: str | None = None


class SensorIndoorAirQuality2(_BaseSchema):
    """Reading for the 'indoorAirQuality' metric. This will only be present if the 'metric'
    property equals 'indoorAirQuality'.
    """

    score: int | None = None


class SensorLivestreamRelatedDevicesItem(_BaseSchema):
    """Schema for SensorLivestreamRelatedDevicesItem."""

    serial: str | None = None
    product_type: str | None = Field(
        default=None, validation_alias="productType", serialization_alias="productType"
    )


class SensorNetwork(_BaseSchema):
    """Information about the network that the sensor and gateway are in."""

    name: str
    id: str


class SensorNoise(_BaseSchema):
    """Noise threshold. 'ambient' must be provided."""

    ambient: SensorAmbient


class SensorNoise2(_BaseSchema):
    """Reading for the 'noise' metric. This will only be present if the 'metric' property equals
    'noise'.
    """

    ambient: SensorAmbient2 | None = None


class SensorPowerFactor(_BaseSchema):
    """Power factor threshold. 'percentage' must be provided."""

    percentage: int


class SensorReadingsItem(_BaseSchema):
    """Schema for SensorReadingsItem."""

    ts: str | None = None
    metric: str | None = None
    apparent_power: SensorApparentPower | None = Field(
        default=None, validation_alias="apparentPower", serialization_alias="apparentPower"
    )
    battery: SensorBattery | None = None
    button: SensorButton | None = None
    co2: SensorCo2 | None = None
    current: SensorApparentPower | None = None
    door: SensorDoor2 | None = None
    downstream_power: SensorDownstreamPower | None = Field(
        default=None, validation_alias="downstreamPower", serialization_alias="downstreamPower"
    )
    frequency: SensorFrequency | None = None
    humidity: SensorHumidity2 | None = None
    indoor_air_quality: SensorIndoorAirQuality2 | None = Field(
        default=None, validation_alias="indoorAirQuality", serialization_alias="indoorAirQuality"
    )
    noise: SensorNoise2 | None = None
    no2: SensorCo2 | None = None
    o3: SensorCo2 | None = None
    pm10: SensorCo2 | None = None
    pm25: SensorCo2 | None = None
    power_factor: SensorBattery | None = Field(
        default=None, validation_alias="powerFactor", serialization_alias="powerFactor"
    )
    real_power: SensorApparentPower | None = Field(
        default=None, validation_alias="realPower", serialization_alias="realPower"
    )
    remote_lockout_switch: SensorRemoteLockoutSwitch | None = Field(
        default=None,
        validation_alias="remoteLockoutSwitch",
        serialization_alias="remoteLockoutSwitch",
    )
    temperature: SensorTemperature2 | None = None
    tvoc: SensorCo2 | None = None
    voltage: SensorFrequency | None = None
    water: SensorWater2 | None = None
    raw_temperature: SensorTemperature2 | None = Field(
        default=None, validation_alias="rawTemperature", serialization_alias="rawTemperature"
    )


class SensorRealPower(_BaseSchema):
    """Real power threshold. 'draw' must be provided."""

    draw: float


class SensorRecipients(_BaseSchema):
    """List of recipients that will receive the alert."""

    emails: list[str] = Field(default_factory=list)
    sms_numbers: list[str] = Field(
        default_factory=list, validation_alias="smsNumbers", serialization_alias="smsNumbers"
    )
    http_server_ids: list[str] = Field(
        default_factory=list, validation_alias="httpServerIds", serialization_alias="httpServerIds"
    )


class SensorRelationships(_BaseSchema):
    """An object describing the relationships defined between the device and other devices."""

    livestream: GetDeviceSensorRelationshipsResponseLivestream | None = None


class SensorRemoteLockoutSwitch(_BaseSchema):
    """Reading for the 'remoteLockoutSwitch' metric. This will only be present if the 'metric'
    property equals 'remoteLockoutSwitch'.
    """

    locked: bool | None = None


class SensorSchedule(_BaseSchema):
    """The sensor schedule to use with the alert profile."""

    id: str | None = None
    name: str | None = None


class SensorSensor(_BaseSchema):
    """Information about the sensor."""

    serial: str | None = None
    name: str
    mac: str


class SensorTemperature(_BaseSchema):
    """Temperature threshold. One of 'celsius', 'fahrenheit', or 'quality' must be provided."""

    celsius: float | None = None
    fahrenheit: float | None = None
    quality: str | None = None


class SensorTemperature2(_BaseSchema):
    """Reading for the 'temperature' metric. This will only be present if the 'metric' property
    equals 'temperature'.
    """

    fahrenheit: float | None = None
    celsius: float | None = None


class SensorThreshold(_BaseSchema):
    """Threshold for sensor readings that will cause an alert to be sent. This object should
    contain a single property key matching the condition's 'metric' value.
    """

    temperature: SensorTemperature | None = None
    humidity: SensorHumidity | None = None
    water: SensorWater | None = None
    door: SensorDoor | None = None
    tvoc: SensorTvoc | None = None
    co2: SensorTvoc | None = None
    pm25: SensorTvoc | None = None
    noise: SensorNoise | None = None
    indoor_air_quality: SensorIndoorAirQuality | None = Field(
        default=None, validation_alias="indoorAirQuality", serialization_alias="indoorAirQuality"
    )
    real_power: SensorRealPower | None = Field(
        default=None, validation_alias="realPower", serialization_alias="realPower"
    )
    apparent_power: SensorRealPower | None = Field(
        default=None, validation_alias="apparentPower", serialization_alias="apparentPower"
    )
    power_factor: SensorPowerFactor | None = Field(
        default=None, validation_alias="powerFactor", serialization_alias="powerFactor"
    )
    current: SensorRealPower | None = None
    voltage: SensorVoltage | None = None
    frequency: SensorVoltage | None = None
    upstream_power: SensorUpstreamPower | None = Field(
        default=None, validation_alias="upstreamPower", serialization_alias="upstreamPower"
    )


class SensorTvoc(_BaseSchema):
    """TVOC concentration threshold. One of 'concentration' or 'quality' must be provided."""

    concentration: int | None = None
    quality: str | None = None


class SensorUpstreamPower(_BaseSchema):
    """Upstream power threshold. 'outageDetected' must be provided and set to true."""

    outage_detected: bool = Field(
        validation_alias="outageDetected", serialization_alias="outageDetected"
    )


class SensorVoltage(_BaseSchema):
    """Voltage threshold. 'level' must be provided."""

    level: float


class SensorWater(_BaseSchema):
    """Water detection threshold. 'present' must be provided and set to true."""

    present: bool


class SensorWater2(_BaseSchema):
    """Reading for the 'water' metric. This will only be present if the 'metric' property equals
    'water'.
    """

    present: bool | None = None


class UpdateDeviceSensorRelationshipsLivestream(_BaseSchema):
    """A role defined between an MT sensor and an MV camera that adds the camera's livestream to
    the sensor's details page. Snapshots from the camera will also appear in alert notifications
    that the sensor triggers.
    """

    related_devices: list[UpdateDeviceSensorRelationshipsLivestreamRelatedDevicesItem] = Field(
        default_factory=list,
        validation_alias="relatedDevices",
        serialization_alias="relatedDevices",
    )


class UpdateDeviceSensorRelationshipsLivestreamRelatedDevicesItem(_BaseSchema):
    """Schema for UpdateDeviceSensorRelationshipsLivestreamRelatedDevicesItem."""

    serial: str


class UpdateDeviceSensorRelationshipsResponse(_BaseSchema):
    """Response for updateDeviceSensorRelationships operation."""

    livestream: GetDeviceSensorRelationshipsResponseLivestream | None = None


class UpdateNetworkSensorAlertsProfileConditionsItem(_BaseSchema):
    """Item schema for conditions."""

    metric: str
    threshold: SensorThreshold
    direction: str | None = None
    duration: int | None = None


class UpdateNetworkSensorAlertsProfileRecipients(_BaseSchema):
    """List of recipients that will receive the alert."""

    emails: list[str] = Field(default_factory=list)
    sms_numbers: list[str] = Field(
        default_factory=list, validation_alias="smsNumbers", serialization_alias="smsNumbers"
    )
    http_server_ids: list[str] = Field(
        default_factory=list, validation_alias="httpServerIds", serialization_alias="httpServerIds"
    )


class UpdateNetworkSensorAlertsProfileResponse(_BaseSchema):
    """Response for updateNetworkSensorAlertsProfile operation."""

    profile_id: str | None = Field(
        default=None, validation_alias="profileId", serialization_alias="profileId"
    )
    name: str | None = None
    schedule: SensorSchedule | None = None
    conditions: list[SensorConditionsItem]
    recipients: SensorRecipients | None = None
    serials: list[str] = Field(default_factory=list)
    include_sensor_url: bool | None = Field(
        default=None, validation_alias="includeSensorUrl", serialization_alias="includeSensorUrl"
    )
    message: str | None = None


class UpdateNetworkSensorAlertsProfileSchedule(_BaseSchema):
    """The sensor schedule to use with the alert profile."""

    id: str | None = None


class UpdateNetworkSensorMqttBrokerResponse(_BaseSchema):
    """Response for updateNetworkSensorMqttBroker operation."""

    mqtt_broker_id: str | None = Field(
        default=None, validation_alias="mqttBrokerId", serialization_alias="mqttBrokerId"
    )
    enabled: bool | None = None



"""Tests for licensing API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from meraki_client import MerakiClient
from tests.generated.conftest import skip_on_unsupported


def test_get_administered_licensing_subscription_entitlements(client: MerakiClient) -> None:
    """Test get_administered_licensing_subscription_entitlements endpoint."""
    with skip_on_unsupported():
        client.licensing.get_administered_licensing_subscription_entitlements()


def test_get_organization_licensing_coterm_licenses(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_licensing_coterm_licenses endpoint."""
    with skip_on_unsupported():
        result = client.licensing.get_organization_licensing_coterm_licenses(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)
